//==================================================================//
//		Engine.cpp													//									
//		Created by O. Reshta (http://www.brigsoft.com), July 2001.	// 
//																	//
//		You can use and modify this file free on terms:				//	
//		1) Don't remove this title.									//
//		2) You will take all responsibility							//
//		   for project functionality after changing this file.		//
//																	//
//		Recommendation: You would insert "Modified  by " comment	//
//						if you will modify this file.				//
//==================================================================//

//Modified by:


#include "stdafx.h"
#include "resource.h"
#include "Engine.h"
#include "Game.h"
#include "stdio.h"
#include "finishdlg.h"
#include "shellapi.h" 
#include <Mmsystem.h>


extern HINSTANCE ghInst;
extern HWND ghWnd;


void DrawColorRect(HDC hDC, RECT* pRect, COLORREF color)
{
	ASSERT(hDC);
	HBRUSH hBr = 0;
	hBr=::CreateSolidBrush(color);
	ASSERT(hBr);
	::FillRect(hDC, pRect, hBr);
	::DeleteObject(hBr);

}

//- CEngine --------------------------------------------------

CEngine::CEngine()
{
	m_nOldAutoTime = m_nOldTime = GetTickCount();
	m_nDeltaTime = 500;					// milliseconds. (500 ms == 0.5s)
	m_nAutoRepDelta	= m_nDeltaTime/5;	

	m_bLockKey			= FALSE;
	m_nTicker = 0;
}

void CEngine::ExecuteTimeTick()
{
	m_nAutoRepDelta	= m_nDeltaTime/5; 
	
	m_nTicker++;


	// Speed up solution.
	if(m_nDeltaTime >250){
		m_nDeltaTime -=25;
	}
	else{
		if(!(m_nTicker%5)){
			if(m_nDeltaTime >10){
				m_nDeltaTime --;
			}
		}
	}
	
	// Next game step calling.
	m_PlayBox.NextTimeStep();

	// To output score in the window title.
	sprintf(m_szBuff,"BSSnake          Length: %04d  Speed: %02d", m_PlayBox.GetSnakeLen(), 500/m_nDeltaTime);
	SendMessage(ghWnd, WM_SETTEXT, 0, (long)m_szBuff);
	
	// Update window to draw new game condition.
	::UpdateWindow(ghWnd);
}

CEngine::TimerProc()
{
	int nTime = GetTickCount();
	if(nTime - m_nOldTime >  m_nDeltaTime){
		// Called once during m_nDeltaTime span.
		ExecuteTimeTick();
		m_nOldTime = nTime;
	}

	if(nTime - m_nOldAutoTime >  m_nAutoRepDelta){
		m_bLockKey = FALSE;
		m_nOldAutoTime = nTime;
	}
}

void CEngine::OnDraw(HDC hDC)
{
	// No comments;-)
	m_PlayBox.Draw(hDC);	
}

LRESULT CEngine::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;

	wmId    = LOWORD(wParam); 
	wmEvent = HIWORD(wParam); 
	// Parse the menu selections:
	switch (wmId)
	{
		case IDM_ABOUT:
		   DialogBox(ghInst, (LPCTSTR)IDD_ABOUTBOX, ghWnd, (DLGPROC)About);
		   break;
		case IDM_EXIT:
		   DestroyWindow(ghWnd);
		   break;
		default:
		   return DefWindowProc(ghWnd, WM_COMMAND, wParam, lParam);
	}

	return 0;
}

// All keyboard control.
LRESULT CEngine::OnKeyDown(WPARAM wParam, LPARAM lParam)
{
	if(m_bLockKey){ 
		return 0;
	}
	
	m_bLockKey = TRUE;

	switch(wParam){
		
		case VK_LEFT:
		case 188: // <
			m_PlayBox.TurnSnakeLeft();
		break;
		
		case VK_RIGHT:
		case 190: // >
			m_PlayBox.TurnSnakeRight();
		break;

		case VK_SPACE:
			if(m_nDeltaTime>10){
				m_nDeltaTime/=1.5;
			}
		break;

		case VK_ESCAPE:
			{
				CFinishDlg dlg;
					switch(dlg.DoModal()){
						case IDOK:
							Init();
						break;
						case IDCANCEL:
							SendMessage(ghWnd, WM_CLOSE,0,0);
						break;
						case IDC_HTTP:
							ShellExecute(NULL, "open", "http://www.brigsoft.com",NULL,"", SW_SHOW);
							SendMessage(ghWnd, WM_CLOSE,0,0);
						break;
					}
			}
		break;

		
	}

	::UpdateWindow(ghWnd);

	return 0;
}

LRESULT CEngine::OnKeyUp(WPARAM wParam, LPARAM lParam)
{
	m_bLockKey = FALSE;
	return 0;
}

void CEngine::Init()
{
	m_nOldAutoTime = m_nOldTime = GetTickCount();
	m_nDeltaTime = 500; 
	m_nAutoRepDelta	= m_nDeltaTime/5; 

	m_bLockKey = FALSE;
	m_nTicker = 0;

	// Game initialization.
	m_PlayBox.Init(); 
}

LRESULT CEngine::OnCreateWindow(WPARAM wParam, LPARAM lParam)
{
	//If the window has non standard title size.
	RECT WinRect, ClientRect;
	::GetWindowRect(ghWnd, &WinRect);
	::GetClientRect(ghWnd, &ClientRect);

	int dH = WinRect.bottom - WinRect.top - ClientRect.bottom +3;
	::SetWindowPos(ghWnd, 0, 0, 0, gnCols*gnCellSize+10, gnRows*gnCellSize+dH, SWP_NOZORDER | SWP_NOMOVE);

	Init();	
	return 0;
}


LRESULT CEngine::WinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	ghWnd = hWnd;

	PAINTSTRUCT ps;
	HDC hdc;


	switch (message) 
	{
		case WM_CREATE:
			return OnCreateWindow(wParam, lParam);
		break;
		
		case WM_COMMAND:
			return OnCommand(wParam, lParam);
			break;
		
		case WM_PAINT:
			hdc = BeginPaint(ghWnd, &ps);
			OnDraw(hdc);
			EndPaint(ghWnd, &ps);
			break;

		case WM_KEYDOWN:
			return OnKeyDown(wParam, lParam);
		break;

		case WM_KEYUP:
			return OnKeyUp(wParam, lParam);
		break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(ghWnd, message, wParam, lParam);
   }

   return 0;
}
