//==================================================================//
//		Game.cpp													//									
//		Created by O. Reshta (http://www.brigsoft.com), July 2001.	// 
//																	//
//		You can use and modify this file free on terms:				//	
//		1) Don't remove this title.									//
//		2) You will take all responsibility							//
//		   for project functionality after changing this file.		//
//																	//
//		Recommendation: You would insert "Modified  by " comment	//
//						if you will modify this file.				//
//==================================================================//

//Modified by:

#include "stdafx.h"
#include "game.h"
#include "resource.h"
#include "finishdlg.h"
#include "engine.h"
#include "shellapi.h" 
#include <Mmsystem.h>


extern HINSTANCE ghInst;
extern HWND ghWnd;
extern CEngine gEngine;
								


// Global array of cells. Game field.
CCell gCell[gnRows][gnCols];

// - CCell: ---------------------------

CCell::CCell()
{
	m_Type = Space;
	m_Dir = Non;
};

HBITMAP CCell::m_hSnakeBmp = 0;


void CCell::Invalidate(int Row, int Col) const
{
	RECT Rect = {Col*gnCellSize, Row*gnCellSize, (Col+1)*gnCellSize, (Row+1)*gnCellSize};
	::InvalidateRect(ghWnd, &Rect, FALSE);
}


void CCell::TurnLeft()
{
	switch (m_Dir){
		case Rt:
			m_Dir = Up;
		break;
		case Up:
			m_Dir = Lf;
		break;
		case Lf:
			m_Dir = Dn;
		break;
		case Dn:
			m_Dir = Rt;
		break;
		default:
			ASSERT(0);
		break;
	}
}
void CCell::TurnRight()
{
	switch (m_Dir){
		case Rt:
			m_Dir = Dn;
		break;
		case Dn:
			m_Dir = Lf;
		break;
		case Lf:
			m_Dir = Up;
		break;
		case Up:
			m_Dir = Rt;
		break;
		default:
			ASSERT(0);
		break;
	}
}


void CCell::Draw(HDC hDC, int Row, int Col) const
{
	RECT Rect = {Col*gnCellSize, Row*gnCellSize, (Col+1)*gnCellSize, (Row+1)*gnCellSize};

	HDC cdc;
	cdc = CreateCompatibleDC(hDC);
	ASSERT(cdc);
	HBITMAP oldhb;
	oldhb = (HBITMAP) SelectObject ( cdc, m_hSnakeBmp);
	
	switch(m_Type){
		
		case Space:
			DrawColorRect(hDC, &Rect, RGB(255,255,255));
		break;
		
		case Head:
			switch(m_Dir){
				case Lf:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 8*gnCellSize, 0, SRCCOPY);
				break;
				case Rt:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 6*gnCellSize, 0, SRCCOPY);
				break;
				case Up:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 9*gnCellSize, 0, SRCCOPY);
				break;
				case Dn:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 7*gnCellSize, 0, SRCCOPY);
				break;
				default:
					ASSERT(0);
				break;
			}
		break;

		case Tail:
			switch(m_Dir){
				case Rt:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 0, 0, SRCCOPY);
				break;
				case Lf:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 2*gnCellSize, 0, SRCCOPY);
				break;
				case Dn:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 3*gnCellSize, 0, SRCCOPY);
				break;
				case Up:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, gnCellSize, 0, SRCCOPY);
				break;
				default:
					ASSERT(0);
				break;
			}		
		break;

		case Body:
			switch(m_Dir){
				case Lf:
				case Rt:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 5*gnCellSize, 0, SRCCOPY);
				break;
				case Up:
				case Dn:
					::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 4*gnCellSize, 0, SRCCOPY);
				break;
				default:
					ASSERT(0);
				break;
			}
		break;

		case BlueMouse: 
			::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 11*gnCellSize, 0, SRCCOPY);
		break;
		case VioletMouse: 
			::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 12*gnCellSize, 0, SRCCOPY);
		break;
		case RedMouse:
			::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 13*gnCellSize, 0, SRCCOPY);
		break;
		case Hedgehog:
			::BitBlt(hDC, Rect.left, Rect.top, gnCellSize, gnCellSize, cdc, 14*gnCellSize, 0, SRCCOPY);
		break;
		default:
			ASSERT(0);
		break;
	}

	SelectObject ( cdc, oldhb );
	DeleteDC(cdc);

}
//- CPnt -----------------------------------------------
void CPnt::MoveToDir(TDir Dir)
{
		switch(Dir){
		case Rt:
			col++;
			if(col >= gnCols){
				throw StopWall();
			}
		break;
		case Up:
			row--;
			if(row <0){
				throw StopWall();
			}
		break;
		case Lf:
			col--;
			if(col < 0){
				throw StopWall();
			}
		break;
		case Dn:
			row++;
			if(row >= gnRows){
				throw StopWall();
			}
		break;
		default:
			ASSERT(0);
		break;
	}

}


// - CSnake: ---------------------------

CSnake::CSnake():m_StartL(3)
{
}

void CSnake::Init()
{
	// To clear the game field.
	for(int i = 0; i<gnRows; i++){
		for(int j=0;j<gnCols;j++){
			gCell[i][j].SetType(CCell::Space);
			gCell[i][j].SetDir(Non);
		}
	}

	// To init snake : set head and tail, then make little growing.
	m_Head.row = 3*gnRows/4;
	m_Head.col = gnCols/2;
	m_Tail.row = m_Head.row + 1;
	m_Tail.col = m_Head.col;

	m_nGrowLen = 0;
	m_nLen = 5;

	gCell[m_Head.row][m_Head.col].SetType(CCell::Head);
	gCell[m_Head.row][m_Head.col].SetDir(Up);
	gCell[m_Tail.row][m_Tail.col].SetType(CCell::Tail);
	gCell[m_Tail.row][m_Tail.col].SetDir(Up);

	gCell[m_Head.row][m_Head.col].Invalidate(m_Head.row, m_Head.col);
	gCell[m_Tail.row][m_Tail.row].Invalidate(m_Tail.row, m_Tail.row);

	for(i =0; i<m_StartL;i++){
		Grow();
	}

	InvalidateRect(ghWnd, NULL, FALSE);
	UpdateWindow(ghWnd);
}

void CSnake::Grow()
{
	gCell[m_Head.row][m_Head.col].Invalidate(m_Head.row, m_Head.col);
	gCell[m_Head.row][m_Head.col].SetType(CCell::Body);

	TDir OldDir = gCell[m_Head.row][m_Head.col].GetDir();

	m_Head.MoveToDir(OldDir);

	gCell[m_Head.row][m_Head.col].SetType(CCell::Head);
	gCell[m_Head.row][m_Head.col].SetDir(OldDir);

	gCell[m_Head.row][m_Head.col].Invalidate(m_Head.row, m_Head.col);

}

void CSnake::Move()
{
	// Move head.
	Grow();

	// Move tail.
	gCell[m_Tail.row][m_Tail.col].Invalidate(m_Tail.row, m_Tail.col);
	
	TDir OldDir = gCell[m_Tail.row][m_Tail.col].GetDir();
	
	gCell[m_Tail.row][m_Tail.col].SetDir(Non);
	gCell[m_Tail.row][m_Tail.col].SetType(CCell::Space);
	
	m_Tail.MoveToDir(OldDir);
	gCell[m_Tail.row][m_Tail.col].SetType(CCell::Tail);

	gCell[m_Tail.row][m_Tail.col].Invalidate(m_Tail.row, m_Tail.col);
}

void CSnake::TestFront()
{
	CPnt TestPnt = m_Head;
	TestPnt.MoveToDir(gCell[m_Head.row][m_Head.col].GetDir());

	CCell::TCellType CellType = gCell[TestPnt.row][TestPnt.col].GetType();

	if(CellType == CCell::Space){
		return;
	}

	// Yahoo!!! Snake is growing.

	switch(CellType){
		case CCell::BlueMouse: 
			m_nGrowLen+=3;
		break;
		case CCell::VioletMouse: 
			m_nGrowLen+=5;
		break;
		case CCell::RedMouse:
			m_nGrowLen+=9;
		break;
		case CCell::Hedgehog:
			throw StopHedgehog();
		break;
		default:
			throw StopBody();
		break;
	}

	PlaySound(MAKEINTRESOURCE(IDR_MOUSEWAVE),ghInst,SND_RESOURCE|SND_ASYNC);


}

void CSnake::TurnLeft()
{
	gCell[m_Head.row][m_Head.col].TurnLeft();
}

void CSnake::TurnRight()
{
	gCell[m_Head.row][m_Head.col].TurnRight();
}

// - CMouse --------------------------------------------

void CMouse::Generate()
{
	m_nTicker ++;

	if(m_nTicker % 10){
		return;
	}

	int nRow = rand()*(gnRows-1)/RAND_MAX;
	int nCol = rand()*(gnCols-1)/RAND_MAX;

	if(gCell[nRow][nCol].GetType() == CCell::Space){

		int nMouse = rand()*5/RAND_MAX;
		switch(nMouse){
			case 1:
				gCell[nRow][nCol].SetType(CCell::BlueMouse);
			break;
			case 2:
				gCell[nRow][nCol].SetType(CCell::VioletMouse);
			break;
			case 3:
				gCell[nRow][nCol].SetType(CCell::RedMouse);
			break;
			default:
				if(!(m_nTicker % 100)){
					gCell[nRow][nCol].SetType(CCell::Hedgehog);
				}
			break;
		}

		gCell[nRow][nCol].Invalidate(nRow, nCol);
	}
}

// - CPlayBox ---------------------------


CPlayBox::CPlayBox()
{
}

CPlayBox::~CPlayBox()
{
	if(CCell::m_hSnakeBmp){
		DeleteObject(CCell::m_hSnakeBmp);
	}
}

void CPlayBox::Init()
{
	// Bitmap is loading here because constructor had be called before instance was created.
	if(CCell::m_hSnakeBmp == 0){
		CCell::m_hSnakeBmp	 = (HBITMAP)LoadBitmap(ghInst,MAKEINTRESOURCE(IDB_SNAKE));
		ASSERT(CCell::m_hSnakeBmp);
	}

	m_Snake.Init();
}

// Game step execution.

void CPlayBox::NextTimeStep()
{
	// Set a amouse ore a hadgehog to the field.
	m_Mouse.Generate();	

	BOOL bCrash = FALSE;

	// Move snake.
	try{
		// Swallow a mouse  ore a hedgehog if it is in the front cell.
		m_Snake.TestFront();

		// Grow one unit if snake must grow.
		if(m_Snake.GetGrowLen()){
			m_Snake.Grow();
			m_Snake.DecGrowLen();
			m_Snake.IncLen();
		}
		// Move snake if it must not grow.
		else{
			m_Snake.Move();
		}
	}

	// Horrible death;-)
	catch(StopWall){
		PlaySound(MAKEINTRESOURCE(IDR_WALLWAVE),ghInst,SND_RESOURCE|SND_ASYNC);
		bCrash = TRUE;
	}
	catch(StopHedgehog){
		PlaySound(MAKEINTRESOURCE(IDR_HEDGEHOGWAVE),ghInst,SND_RESOURCE|SND_ASYNC);
		bCrash = TRUE;
	}
	catch(StopBody){
		PlaySound(MAKEINTRESOURCE(IDR_HEDGEHOGWAVE),ghInst,SND_RESOURCE|SND_ASYNC);
		bCrash = TRUE;
	}
	catch(...){
		ASSERT(0);
	}

	// Death dialog.
	if(bCrash){
		CFinishDlg dlg;
		switch(dlg.DoModal()){
			case IDOK:
				gEngine.Init();
			break;
			case IDCANCEL:
				SendMessage(ghWnd, WM_CLOSE,0,0);
			break;
			case IDC_HTTP:
				ShellExecute(NULL, "open", "http://www.brigsoft.com",NULL,"", SW_SHOW);
				SendMessage(ghWnd, WM_CLOSE,0,0);
			break;
		}
	}

}


void CPlayBox::Draw(HDC hDC)
{
	
	for(int i = 0; i<gnRows; i++){
		for(int j=0;j<gnCols;j++){
			gCell[i][j].Draw(hDC,i,j);
		}
	}


}